@@ -13,8 +13,9 @@
 #'   Default: 20
 #' @param max_visit_sd standard deviation of maximum number of visits of each
 #'   patient, Default: 4
-#' @param ae_per_visit_mean mean ae per visit per patient, Default: 0.5
-#' @param ae_rates vector with visit-specific ae rates, Default: Null
+#' @param ae_per_visit_mean mean event per visit per patient, Default: 0.5
+#' @param ae_rates vector with visit-specific event rates, Default: Null
+#' @param event_names vector, contains the event names, default = "ae"
 #' @return tibble with columns site_number, patnum, is_ur, max_visit_mean,
 #'   max_visit_sd, ae_per_visit_mean, visit, n_ae
 #' @details maximum visit number will be sampled from normal distribution with
@@ -33,57 +34,87 @@
 #' @rdname sim_test_data_study
 #' @export
 sim_test_data_study <- function(n_pat = 1000,
-                                n_sites = 20,
-                                frac_site_with_ur = 0,
-                                ur_rate = 0,
-                                max_visit_mean = 20,
-                                max_visit_sd = 4,
-                                ae_per_visit_mean = 0.5,
-                                ae_rates = NULL
+                                 n_sites = 20,
+                                 frac_site_with_ur = 0,
+                                 ur_rate = 0,
+                                 max_visit_mean = 20,
+                                 max_visit_sd = 4,
+                                 ae_per_visit_mean = c(0.5),
+                                 ae_rates = c(NULL),
+                                 event_names = list("ae")
 ) {
 
+
+  if (length(event_names) != length(ae_per_visit_mean)) {
+    stop(paste0("Number of events named (", length(event_names),
+                ") doesn't equal the number of events per visit means submitted (", length(ae_per_visit_mean), ")"))
+  }
+
+  if (!(is.null(ae_rates))) {
+    if (is.numeric(ae_rates) && length(event_names) > 1) {
+      stop(paste0("ae_rates should be entered as a list (containing arrays) when the number of events is > 1"))
+    }
+    if (is.list(ae_rates) && length(ae_rates) != length(event_names)) {
+      stop(paste0("Number of events named (", length(event_names),
+                                   ") doesn't equal the number of events rates submitted (", length(ae_rates), ")"))
+    }
+    }
   # construct patient ae sample function
   # supports constant and non-constant ae rates
-
   f_sim_pat <- function(vs_max, vs_sd, is_ur) {
-
+    colnames <- c(paste0(event_names, "_per_visit_mean"), "visit",  paste0("n_", event_names))
     if (! any(c(is.null(ae_rates), is.na(ae_rates)))) {
-
       if (is_ur) {
-        ae_rates <- ae_rates * (1-ur_rate) # nolint
+        ae_rates <- ae_rates * (1 - ur_rate)
       }
 
       f_sample_ae <- function(max_visit) {
 
         # extrapolate missing ae rates by extending last rate
-        fill <- rep(ae_rates[length(ae_rates)], max_visit)
-        fill[seq_along(ae_rates)] <- ae_rates
+        if (is.list(ae_rates)) {
+
+          fill <- map(ae_rates, .f = ~(rep(.x[length(.x)], max_visit)))
+          for (x in seq_along(ae_rates)) {
+
+            fill[[x]][seq_along(ae_rates[[x]])] <- ae_rates[[x]]
+            }
+          }else {
+
+            fill <- rep(ae_rates[length(ae_rates)], max_visit)
+            fill[seq_along(ae_rates)] <- ae_rates
+          }
+
+
         ae_rates <- fill
 
-        aes <- integer(0)
 
-        for (i in seq(1, max_visit)) {
-          ae <- rpois(1, ae_rates[i])
-          aes <- c(aes, ae)
+        for (x in seq_along(event_names)){
+
+          aes_temp <- numeric(0)
+
+          for (i in seq(1, max_visit)) {
+            ae <- rpois(1, ifelse(is.list(ae_rates), ae_rates[[x]][i], ae_rates[i]))
+            aes_temp <- c(aes_temp, ae)
+          }
+          if (x == 1) (aes <- aes_temp)
+          else (aes <- list(aes, aes_temp))
         }
 
         return(aes)
-
       }
 
-      ae_per_visit_mean <- mean(ae_rates)
+      ae_per_visit_mean <- if (is.list(ae_rates)) map(ae_rates, mean) else (mean(ae_rates))
 
     } else {
 
-      if (is_ur) {
-        ae_per_visit_mean <- ae_per_visit_mean * (1-ur_rate) # nolint
-      }
+        if (is_ur) {
+          ae_per_visit_mean <- ae_per_visit_mean * (1 - ur_rate)
+        }
 
-      f_sample_ae <- function(max_visit) {
-
-        rpois(max_visit, ae_per_visit_mean)
-
-      }
+        f_sample_ae <- function(max_visit) {
+          ae_per_visit_mean %>%
+            map(rpois, n = max_visit)
+        }
 
     }
 
@@ -92,17 +123,15 @@ sim_test_data_study <- function(n_pat = 1000,
       .f_sample_ae_per_visit = f_sample_ae
     )
 
-    tibble(
-      ae_per_visit_mean = ae_per_visit_mean,
-      visit = seq(1, length(aes)),
-      n_ae = aes,
-    )
 
+    sim_pat <- data.frame(ae_per_visit_mean = as.list(ae_per_visit_mean),
+                          visit = seq(1, length(aes[[1]])),
+                          n_ae = aes)
+
+    names(sim_pat) <- colnames
+    return(sim_pat)
   }
 
-
-
-
   tibble(patnum = seq(1, n_pat)) %>%
     mutate(patnum = str_pad(patnum, width = 6, side = "left", pad = "0"),
            patnum = paste0("P", patnum),
@@ -117,8 +146,8 @@ sim_test_data_study <- function(n_pat = 1000,
                            .data$max_visit_sd,
                            .data$is_ur),
                       f_sim_pat
-                      )
-           ) %>%
+           )
+    ) %>%
     unnest("aes")
 
 }
@@ -147,8 +176,9 @@ sim_test_data_patient <- function(.f_sample_max_visit = function() rnorm(1, mean
   max_visit <- as.integer(.f_sample_max_visit())
   max_visit <- ifelse(max_visit < 1, 1, max_visit)
   aes <- .f_sample_ae_per_visit(max_visit)
-  cum_aes <- cumsum(aes)
 
+  if (is.list(aes)) (cum_aes <- aes %>% map(cumsum))
+  else (cum_aes <- list(aes) %>% map(cumsum))
   return(cum_aes)
 }
 
@@ -174,7 +204,11 @@ sim_scenario <- function(n_ae_site, n_ae_study, frac_pat_with_ur, ur_rate) {
     return(list(n_ae_site = n_ae_site, n_ae_study = n_ae_study))
   }
 
-  if (frac_pat_with_ur > 1) frac_pat_with_ur <- 1
+  if (frac_pat_with_ur > 1) {
+    frac_pat_with_ur <- 1
+    #Warning message if frac_pat_with_ur > 1
+    warning("Fraction of patients with underreporting is greater than 1.\n  The fraction has been changed to 1")
+  }
 
   n_pat_site <- length(n_ae_site)
   n_pat_study <- length(n_ae_study)
@@ -630,11 +664,9 @@ sim_test_data_portfolio <- function(df_config, df_ae_rates = NULL, parallel = FA
         ae_rates = map(.data$data, "ae_rate")
       ) %>%
       select(- "data")
-
     df_config <- df_config %>%
       inner_join(df_ae_rates, by = "study_id")
   }
-
   # exec --------------------------
 
   if (parallel) {
@@ -682,6 +714,8 @@ sim_test_data_portfolio <- function(df_config, df_ae_rates = NULL, parallel = FA
     progress = progress
   )
 
+
+
   df_portf <- df_config_sim %>%
     unnest("sim") %>%
     select(- c("n_pat", "ae_rates")) %>%
@@ -696,7 +730,6 @@ sim_test_data_portfolio <- function(df_config, df_ae_rates = NULL, parallel = FA
       )
     ) %>%
     ungroup()
-
   return(df_portf)
 }
 
@@ -902,7 +935,7 @@ get_portf_perf <- function(df_scen, stat = "prob_low_prob_ur", fpr = c(0.001, 0.
 
   }
 
-  stat_at_0 <- df_scen %>% # nolint
+  stat_at_0 <- df_scen %>% #nolint
     filter(.data$ur_rate == 0, .data$frac_pat_with_ur == 0) %>%
     pull(.data[[stat]])
 
@@ -997,3 +1030,34 @@ sim_ur <- function(df_visit, study_id, site_number, ur_rate) {
   bind_rows(df_visit_study, df_visit_site_rem)
 
 }
+
+#' @title simulate test data events
+#' @description generates multi-event data using sim_test_data_study()
+#' @param n_pat integer, number of patients, Default: 100
+#' @param n_sites integer, number of sites, Default: 5
+#' @param ae_per_visit_mean mean event per visit per patient, Default: 0.5
+#' @param ae_rates vector with visit-specific event rates, Default: Null
+#' @param event_names vector, contains the event names, default = "ae"
+#' @return tibble with columns site_number, patnum, is_ur, max_visit_mean,
+#'   max_visit_sd, visit, and event data (events_per_visit_mean and n_events)
+#' @export
+sim_test_data_events <- function(
+    n_pat = 100,
+    n_sites = 5,
+    ae_per_visit_mean = 0.5,
+    ae_rates = c(NULL),
+    event_names = list("ae")) {
+
+  set.seed(1)
+  df_visit3 <- sim_test_data_study(n_pat = 100, n_sites = n_sites,
+                                   frac_site_with_ur = 0.4, ur_rate = 0.6,
+                                   ae_per_visit_mean = ae_per_visit_mean, event_names = event_names)
+  df_visit3$study_id <- "A"
+  set.seed(2)
+  df_visit4 <- sim_test_data_study(n_pat = 100, n_sites = n_sites,
+                                   frac_site_with_ur = 0.2, ur_rate = 0.1,
+                                   ae_per_visit_mean = ae_per_visit_mean, event_names = event_names)
+  df_visit4$study_id <- "B"
+  df_visit_events_test <- dplyr::bind_rows(df_visit3, df_visit4)
+  return(df_visit_events_test)
+}
